'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

require('isomorphic-fetch');

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var mimeLib = {
  json: 'application/json',
  form: 'multipart/form-data',
  html: 'text/html',
  xml: 'application/xml',
  text: 'text/plain'
};

var Hifetch = function () {
  function Hifetch(opt) {
    _classCallCheck(this, Hifetch);

    this._config = Object.assign({}, Hifetch.defaultConfig, opt);
    var _config = this._config;
    var query = _config.query;
    var method = _config.method;
    var responseType = _config.responseType;
    var originDataType = _config.dataType;
    var data = _config.data;
    var headers = _config.headers;
    var jwtToken = _config.jwtToken;
    var enableCookies = _config.enableCookies;
    var disableCORS = _config.disableCORS;

    var fetchOpt = this._fetchOpt = {};
    if (disableCORS) {
      fetchOpt.mode = 'cors';
    }
    if (enableCookies) {
      fetchOpt.credentials = disableCORS ? 'same-origin' : 'include';
    }
    var moreHeaders = {
      Accept: /\//.test(responseType) ? responseType : mimeLib[responseType]
    };
    if (jwtToken) {
      moreHeaders['Authorization'] = 'Bearer ' + jwtToken;
    }
    var dataType = (/\//.test(originDataType) ? originDataType : mimeLib[originDataType]) || Hifetch.defaultConfig.dataType;
    var validMethod = method.toLowerCase();
    if (validMethod === 'post') {
      var dataString = void 0;
      if ((typeof data === 'undefined' ? 'undefined' : _typeof(data)) === 'object') {
        if (dataType === Hifetch.defaultConfig.dataType) {
          dataString = _qs2.default.stringify(data);
        } else if (dataType.indexOf('json') !== -1) {
          dataString = JSON.stringify(data);
        }
      }
      Object.assign(fetchOpt, {
        method: 'POST',
        headers: Object.assign({}, headers, moreHeaders, {
          'Content-Type': dataType
        }),
        body: dataString || data
      });
    } else {
      Object.assign(fetchOpt, {
        headers: Object.assign({}, headers, moreHeaders)
      });
    }
    this._queryString = query ? _qs2.default.stringify(query) : '';
  }

  _createClass(Hifetch, [{
    key: 'send',
    value: function send() {
      var _this = this;

      var _config2 = this._config;
      var url = _config2.url;
      var timeout = _config2.timeout;
      var validateStatus = _config2.validateStatus;
      var parser = _config2.parser;
      var handler = _config2.handler;
      var success = _config2.success;
      var error = _config2.error;

      var current = this._current = uuid();
      return new Promise(function (resolve, reject) {
        if (timeout) {
          setTimeout(function () {
            if (current === _this._current) {
              reject('timeout');
            }
          }, timeout);
        }
        return fetch(_this._queryString ? url + '?' + _this._queryString : url, _this._fetchOpt).then(function (res) {
          if (current === _this._current) {
            resolve(res);
          }
        }).catch(function (res) {
          if (current === _this._current) {
            reject(res);
          }
        });
      }).then(function (response) {
        if (validateStatus(response.status)) {
          return response;
        }
        var err = new Error(response.statusText);
        err.response = response;
        throw err;
      }).then(parser).then(function (data) {
        try {
          if ((typeof data === 'undefined' ? 'undefined' : _typeof(data)) === 'object' && data.status) {
            return error({
              status: 3,
              message: '[REMOTE ERROR] status: ' + data.status + ' message: ' + data.message,
              data: data
            });
          }
          var result = handler ? handler(data) : data;
          return success(result);
        } catch (err) {
          return error({
            status: 4,
            message: '[CUSTOM JS ERROR] ' + clearErrorMessage(err),
            error: err
          });
        }
      }).catch(function (err) {
        if (err === 'timeout') {
          return error({
            status: 5,
            message: '[TIMEOUT ERROR] limit: ' + timeout / 1000 + 's',
            timeout: timeout
          });
        }
        if (err.status) {
          return error(err);
        }

        var _ref = err.response || {};

        var httpStatus = _ref.status;
        var httpStatusText = _ref.statusText;

        if (httpStatus) {
          return error({
            status: 2,
            message: '[FETCH ERROR] ' + httpStatus + ' ' + httpStatusText,
            httpStatus: httpStatus,
            httpStatusText: httpStatusText
          });
        }
        return error({
          status: 1,
          message: '[INTERNAL JS ERROR] ' + clearErrorMessage(err),
          error: err
        });
      });
    }
  }, {
    key: 'cancel',
    value: function cancel() {
      this._current = null;
    }
  }, {
    key: 'error',
    value: function error(customError) {
      var error = this._config.error;

      this._current = null;
      return error({
        status: 6,
        message: '[MANUAL ERROR] ' + clearErrorMessage(customError),
        customError: customError
      });
    }
  }]);

  return Hifetch;
}();

Hifetch.defaultConfig = {
  url: '',
  method: 'get',
  query: null,
  data: '',
  dataType: 'application/x-www-form-urlencoded',
  jwtToken: '',
  headers: {},
  enableCookies: false,
  disableCORS: false,
  validateStatus: function validateStatus(status) {
    return status >= 200 && status < 300;
  },
  parser: function parser(response) {
    return response.json();
  },

  responseType: 'application/json',
  timeout: 0,
  handler: null,
  success: function success(res) {
    return res;
  },
  error: function error(res) {
    return Promise.reject(res);
  }
};


function uuid() {
  return new Date().getTime() * 1000 + Math.floor(Math.random() * 1000);
}

function clearErrorMessage(err) {
  return err.toString(); // .replace(/.+:\s*/, '');
}

function hifetch(opt) {
  return new Hifetch(opt);
}

hifetch.Class = Hifetch;

exports.default = hifetch;
//# sourceMappingURL=index.js.map